/* lsh, an implementation of the ssh protocol
 *
 * Copyright (C) 1998 Niels Mller
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _BIGNUM_H
#define _BIGNUM_H

#include <config.h>

/* Needed for the declaration of mpz_out_str */
#include <stdio.h>

#ifdef HAVE_GMP_H
#include <gmp.h>
#else
#error No gmp.h!
#endif

#include "common.h"


void bignum_parse_s(mpz_t n, word32 length, const word8 *data);
void bignum_parse_u(mpz_t n, word32 length, const word8 *data);

word32 bignum_format_s(const mpz_t n, word8 *data);
word32 bignum_format_s_length(const mpz_t n);

word32 bignum_format_u(const mpz_t n, word8 *data);
word32 bignum_format_u_length(const mpz_t n);

void bignum_write(mpz_t n, unsigned length, word8 *data);

/* Generates a random number in the interval 0 <= x < n */
void bignum_random(mpz_t x, mpz_t n);
void bignum_random_size(mpz_t x, unsigned bits);
void bignum_random_prime(mpz_t x, unsigned bits);

unsigned long bignum_small_factor(mpz_t n, int limit);
void bignum_next_prime(mpz_t p, mpz_t n, int count, int prime_limit);

#endif /* _BIGNUM_H */
