/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>
#include <unistd.h>

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <netdb.h>

#ifdef __USE_MCRYPT
#ifndef _MCRYPT_H
#define _MCRYPT_H
#include <mcrypt.h>
#endif
#endif


#include "common.h"
#include "conf.h"
#include "thread.h"
#include "log.h"
#include "bignum.h"
#include "crypt.h"
#include "packet.h"
#include "node.h"
#include "caller.h"


caller_t *caller_create(int sockd) {
	caller_t *caller;

	if ((caller = _malloc(sizeof(caller_t))) == NULL) {
		ERROR("could not malloc space for a new caller");
		return NULL;
	}
	bzero(caller,sizeof(caller_t));
	
	object_init(caller,caller_free,caller);
	thread_init(caller,caller_run,caller);

	caller->link		= NULL;

	__objects_created++;

	return caller;
} /* end node_create */


void caller_free(void *o) {
	caller_t *call = (caller_t *)o;
	
	DUMP("freeing caller %p",(void *)call);

	thread_destroy(call);
	object_destroy(call);

	if (call->link != NULL)
		object_release(call->link);

	_free(call);

	__objects_freed++;
	
	return; /* void */
} /* end caller_free */


void caller_run(void *o) {
	caller_t *call = (caller_t *)o;
	int i;

	for (i = 0; i < CALLER_MAX_NODES; i++) {
		call->nodes[i].hopid = i;
		node_pick(&call->nodes[i]);
	}

	/*** DEBUGGING TEST RUN ***/
	if (caller_link_connect(call)) {
		call->nodes[0].channel = channel_create(call->link);
		object_retain(call->nodes[0].channel);
		if (channel_initiate(call->nodes[0].channel) == 0)
			DEBUG("channel established");
		else
			DEBUG("channel failed");
		
		object_release(call->nodes[0].channel);
	}

	if (call->link != NULL)
		object_release(call->link);

} /* caller_run */



bool caller_proxy_initiate(caller_t *caller) {

	return false;
} /* end caller_proxy_initiate */


bool caller_link_connect(caller_t *call) {
	int sd;
	struct sockaddr_in addr;
	node_ctx *n = &call->nodes[0];

	if ((sd = socket(PF_INET,SOCK_STREAM,IPPROTO_TCP)) < 0) {
		ERROR("could not create socket: %s",strerror(errno));
		return false;
	}

	link_setsockopts(sd);

	addr.sin_family	= AF_INET;
	addr.sin_port	= htons(n->port);

	switch (inet_pton(AF_INET,n->host,&addr.sin_addr)) {
		case 0:
			WARN("could not convert address %s",n->host);
			return false;
		case -1:
			WARN("could not convert address %s: %s",n->host,strerror(errno));
			return false;
	}


	if (connect(sd, (struct sockaddr *)&addr,(socklen_t)sizeof(struct sockaddr_in)) != 0) {
		ERROR("could not connect to %s:%i: %s",n->host,n->port,strerror(errno));
		return false;
	}	

	if ((call->link = link_create(sd)) == NULL) {
		return false;
	} else {
		object_retain(call->link);
		call->link->iscaller = true;
	}

	if (link_initiate(call->link) != 0) {
		return false;
	}


	return true;

} /* end caller_link_connect */




void node_pick(node_ctx *info) {
	extern conf_t conf;
	unsigned char b;
	unsigned char *mode;
	unsigned char modes[6][2] = {
		{RIJNDAEL_128,SHA1_160},{RIJNDAEL_128,TIGER_192},
		{TWOFISH_128,SHA1_160},{TWOFISH_128,TIGER_192},
		{SERPENT_128,SHA1_160},{SERPENT_128,TIGER_192}
	};

	//rand_get(&b,1);
	//mode = modes[b%6];
	//DEBUG("using cipher: %s; using hash: %s;",CIPHER_NAME(mode[0]),HASH_NAME(mode[1]));

	object_lock(&conf);

	strcpy(info->host,"127.0.0.1");
	info->port = (uint16_t)conf.port;
	//info->port = (uint16_t)8080;

	object_unlock(&conf);

	return; /* void */

} /* end node_pick */

