/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <pthread.h>

#include "options.h"
#include "conf.h"
#include "log.h"


/* not thread-safe if conf reloaded, but shouldn't harm anything */
bool __log_level(int level) {

	if (__conf_initialized) {
		switch (level) {
			case ANET_LOG_ERROR:
				return conf.enable_error;
			case ANET_LOG_WARN:
				return conf.enable_warn;
			case ANET_LOG_INFO:	
				return conf.enable_info;
			case ANET_LOG_DEBUG:
				return conf.enable_debug;
			case ANET_LOG_DUMP:
				return conf.enable_dump;
			case ANET_LOG_SOURCE:
				return conf.enable_track_source;
			case ANET_LOG_FUNC:
				return conf.enable_track_func;
			case ANET_LOG_THREAD:
				return conf.enable_track_thread;
			default:
				return false;
		}
	} else {
		return (level < DEFAULT_VERBOSITY)? true : false;
	}
}


void log_init() {

	if (conf.daemon) {
		openlog(PACKAGE,LOG_CONS | LOG_PID,LOG_DAEMON);
		__log_initialized = true;
		return; /* void */
	} else {
		//openlog(PACKAGE,LOG_CONS | LOG_PERROR | LOG_PID,LOG_DAEMON);
		//__log_initialized = true;
		return; /* void */
	}
}


void log(int type,char *file,int line,char *func,char *format,...) {
	va_list args;
	int level;
	char str[255] = {0};

	/* allow the compiler to optimize out irrelevent options */
	if (ENABLE_TRACK_SOURCE && ENABLE_TRACK_FUNC && ENABLE_TRACK_THREAD) {
		snprintf(str,sizeof(str),"%s:%i:%s:%i:",file,line,func,(int)pthread_self());
	} else if (ENABLE_TRACK_SOURCE && ENABLE_TRACK_FUNC) {
		snprintf(str,sizeof(str),"%s:%i:%s:",file,line,func);
	} else if (ENABLE_TRACK_SOURCE && ENABLE_TRACK_THREAD) {
		snprintf(str,sizeof(str),"%s:%i:%i:",file,line,(int)pthread_self());
	} else if (ENABLE_TRACK_FUNC && ENABLE_TRACK_THREAD) {
		snprintf(str,sizeof(str),"%s:%i:",func,(int)pthread_self());
	} else if (ENABLE_TRACK_SOURCE) {
		snprintf(str,sizeof(str),"%s:%i:",file,line);
	} else if (ENABLE_TRACK_FUNC) {
		snprintf(str,sizeof(str),"%s:",func);
	} else if (ENABLE_TRACK_THREAD) {
		snprintf(str,sizeof(str),"%i:",(int)pthread_self());
	}

	switch(type) {
		case ANET_LOG_ERROR:
			if (ENABLE_ERROR) {
				level = LOG_ERR;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","error: ");
				break;
			} else
				return;
		case ANET_LOG_WARN:
			if (ENABLE_WARN) {
				level = LOG_WARNING;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","warn: ");
				break;
			} else
				return;
		case ANET_LOG_INFO:
			if (ENABLE_INFO) {
				level = LOG_INFO;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","info: ");
				break;
			} else
				return;
		case ANET_LOG_DEBUG:
			if (ENABLE_DEBUG) {
				level = LOG_DEBUG;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","debug: ");
				break;
			} else
				return;
		case ANET_LOG_DUMP:
			if (ENABLE_DUMP) {
				level = LOG_DEBUG;
				snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"%s","dump: ");
				break;
			} else
				return;
		default:
			snprintf(&str[strlen(str)],sizeof(str) - strlen(str),"log error: invalid argument specified, %i, in file %s, line %i",type,file,line);
			level = LOG_ERR;
	}

	va_start(args,format);
	 /* decrease size argument by 1? are all vsnprintf()
	  * implementations guaranteed to NUL terminate? then
	  * we'll have to make sure str[254] is NUL as well.
	  */
	vsnprintf(&str[strlen(str)],sizeof(str) - strlen(str),format,args);
	va_end(args);

	if (__log_initialized)
		syslog(level,str);
	else
		fprintf(stderr,"%s\n",str);
		
	return;
}
