/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _LOG_H
#define _LOG_H 1

#include <stdlib.h>
#include <syslog.h>
#include <signal.h>

#include "options.h"
#include "conf.h"


#define ENABLE_ERROR	1
#define ENABLE_WARN	__log_level(ANET_LOG_WARN)

#ifdef __DISABLE_INFO
#define ENABLE_INFO	0
#else
#define ENABLE_INFO	__log_level(ANET_LOG_INFO)
#endif

#ifdef __ENABLE_DEBUG
#define ENABLE_DEBUG	__log_level(ANET_LOG_DEBUG)
#define ENABLE_DUMP	__log_level(ANET_LOG_DUMP)
#else
#define ENABLE_DEBUG	0
#define ENABLE_DUMP	0
#endif

#ifdef __ENABLE_TRACK_SOURCE
#define ENABLE_TRACK_SOURCE	__log_level(ANET_LOG_SOURCE)
#else
#define ENABLE_TRACK_SOURCE	0
#endif

#ifdef __ENABLE_TRACK_FUNC
#define ENABLE_TRACK_FUNC	__log_level(ANET_LOG_FUNC)
#else
#define ENABLE_TRACK_FUNC	0
#endif

#ifdef __ENABLE_TRACK_THREAD
#define ENABLE_TRACK_THREAD	__log_level(ANET_LOG_THREAD)
#else
#define ENABLE_TRACK_THREAD	0
#endif


/*
 * Error Routines
 *
 */
#define ANET_LOG_ERROR	0
#define ANET_LOG_WARN	1
#define ANET_LOG_INFO	2
#define ANET_LOG_DEBUG	3
#define ANET_LOG_DUMP	4
#define ANET_LOG_SOURCE	5
#define ANET_LOG_FUNC	6
#define ANET_LOG_THREAD	7


#define	ERROR(...)	log(ANET_LOG_ERROR,__FILE__,__LINE__,__func__,__VA_ARGS__)
#define	WARN(...)	log(ANET_LOG_WARN,__FILE__,__LINE__,__func__,__VA_ARGS__)

#ifdef __DISABLE_INFO
#define	INFO(...)
#else
#define	INFO(...)	log(ANET_LOG_INFO,__FILE__,__LINE__,__func__,__VA_ARGS__)
#endif

#ifdef __ENABLE_DEBUG
#define DEBUG(...)	log(ANET_LOG_DEBUG,__FILE__,__LINE__,__func__,__VA_ARGS__)
#define DUMP(...)	log(ANET_LOG_DUMP,__FILE__,__LINE__,__func__,__VA_ARGS__)
#else
#define	DEBUG(...)
#define DUMP(...)
#endif

#define ABORT(...)	do { ERROR(__VA_ARGS__); abort(); } while(0);
#define ASSERT(expr,...) \
			do { if (!((expr))) ABORT(__VA_ARGS__); } while(0);

sig_atomic_t __log_initialized;

bool __log_level(int);

void log_init(void);
void log(int type,char *file,int line,char *func,char *format,...);


#endif /* _LOG_H */
