/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include "common.h"
#include "options.h"
#include "conf.h"
#include "log.h"

void options_init() {
	extern options_t options;

	return; /* void */
}

void options_parse(int argc,char *argv[]) {
	extern options_t options;
	int c;

	static struct option getopt_options[] = {
			{"config",		1, NULL, 'c'},
			{"port",		1, NULL, 'p'},
			{"listen",		1, NULL, 'l'},
			{"random",		1, NULL, 'r'},
			{"daemon",		0, NULL, 'd'},
			{"foreground",		0, NULL, 'f'},
			{"user",		1, NULL, 'u'},
			{"group",		1, NULL, 'g'},
			{"tty",			1, NULL, 't'},
			{"verbose",		0, NULL, 'w'},
			{"help",		0, NULL, 1},
			{"license",		0, NULL, 2},
			{"version",		0, NULL, 3},
			{"debug",		0, NULL, 4},
			{"track-source",	0, NULL, 5},
			{"track-func",		0, NULL, 6},
			{"track-thread",	0, NULL, 7},
			{"track-all",		0, NULL, 8}
	};

	options.command		= *argv;
	options.config		= NULL;
	options.port		= 0;
	options.listen		= NULL;
	options.random		= NULL;
	options.daemon		= false;
	options.foreground	= false;
	options.user		= NULL;
	options.group		= NULL;
	options.usetty		= false;
	options.tty		= NULL;
	options.verbosity	= 0;
	options.help		= false;
	options.license		= false;
	options.version		= false;
	options.debug		= false;
	options.track_source	= false;
	options.track_func	= false;
	options.track_thread	= false;

	while ((c = getopt_long(argc,argv,"+c:r:p:l:dfu:g:t:v",getopt_options,NULL)) != -1) {
		switch (c) {
			case 'c':
				if ((options.config = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options.command);
					perror("an error occured parsing options");
					exit(EXIT_FAILURE);
				}
				break;
			case 'p':
				options.port = (unsigned short)atoi(optarg);
				break;
			case 'l':
				if ((options.listen = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options.command);
					perror("an error occured parsing options");
					exit(EXIT_FAILURE);
				}
				break;
			case 'r':
				if ((options.random = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options.command);
					perror("an error occured parsing options");
					exit(EXIT_FAILURE);
				}
				break;
			case 'd':
				options.daemon = true;
				break;
			case 'f':
				options.foreground = true;
				break;
			case 'u':
				if ((options.user = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options.command);
					perror("an error occured parsing options");
					exit(EXIT_FAILURE);
				}
				break;
			case 'g':
				if ((options.group = (char *)strdup(optarg)) == NULL) {
					fprintf(stderr,"%s: ",options.command);
					perror("an error occured parsing options");
					exit(EXIT_FAILURE);
				}
				break;
			case 't':
				options.usetty = true;
				if (optarg != NULL) {
					if ((options.tty = (char *)strdup(optarg)) == NULL) {
						fprintf(stderr,"%s: ",options.command);
						perror("an error occured parsing options");
						exit(EXIT_FAILURE);
					}
					fprintf(stderr,"%s: cannot take a tty argument yet.\n",options.command);
				}
				break;
			case 'v':
				options.verbosity = MIN((options.verbosity+1),MAX_VERBOSITY);
				break;
			case 'w':
				options.verbosity = MAX_VERBOSITY;
				break;
			case 1:
				options.help = true;
				break;
			case 2:
				options.license = true;
				break;
			case 3:
				options.version = true;
				break;
			case 4:
				options.debug = true;
				break;
			case 5:
				options.track_source = true;
				break;
			case 6:
				options.track_func = true;
				break;
			case 7:
				options.track_thread = true;
				break;
			case 8:
				options.track_source = true;
				options.track_func = true;
				options.track_thread = true;
				break;
			case '?':
			default:
				/* getopt prints to stderr */
				exit(EXIT_FAILURE);
		}

	}

	if ((options.daemon) && (options.foreground)) {
		fprintf(stderr,"%s: cannot use foreground and daemon switches together\n",*argv);
		exit(EXIT_FAILURE);
	}
		
}

