/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _COMMON_H
#define _COMMON_H 1

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


/* protocol version info
 * these should be set in configure.in somehow
 */
#define PROTOCOL_MAJOR	0
#define PROTOCOL_MINOR	0
#define PROTOCOL_PATCH	1


/* boolean types */
#if __STDC_VERSION__ >= 199901L || HAVE_STDBOOL_H
#include <stdbool.h>
#else
typdef enum {
	true = 1, false = 0
} _Bool;
#define true	true
#define false	false
#define bool	_Bool
#define __bool_true_false_are_define	1
#endif /* boolean types */


/* int types */
#if HAVE_INTTYPES_H
#include <inttypes.h>
#else

/* 64 bit uint */
#if SIZEOF_UNSIGNED_LONG_INT == 8
typedef unsigned long int uint64_t;
#elif SIZEOF_UNSIGNED_LONG_LONG_INT == 8
typedef unsigned long long int uint64_t;
#else
#error "Cannot find a 64 bit integer in your system, sorry."
#endif /* 64 bit uint */

/* 32 bit uint */
#if SIZEOF_UNSIGNED_LONG_INT == 4
typedef unsigned long int uint32_t;
#elif SIZEOF_UNSIGNED_INT == 4
typedef unsigned int uint32_t;
#else
#error "Cannot find a 32 bit integer in your system, sorry."
#endif /* 32 bit uint */

/* 16 bit uint */
#if SIZEOF_UNSIGNED_INT == 2
typedef unsigned int uint16_t;
#elif SIZEOF_UNSIGNED_SHORT_INT == 2
typedef unsigned short int uint16_t;
#else
#error "Cannot find a 16 bit integer in your system, sorry."
#endif /* 16 bit uint */

/* 8 bit uint */
#if SIZEOF_UNSIGNED_CHAR == 1
typedef unsigned char uint8_t;
#else
#error "Cannot find a 8 bit integer in your system, sorry."
#endif /* 8 bit uint */

#endif /* int types */

#define word64	uint64_t
#define word32	uint32_t
#define word16	uint16_t
#define word8	uint8_t

#ifdef HAVE_BZERO
#define bzero(s,c) bzero((s),(c))
#elif HAVE_MEMSET
#define bzero(s,c) memset((s),0,(c))
#else
#error "No bzero or memset found in your system, sorry."
#endif /* bzero/memset */

#define unless(cond) if(!(cond))

#ifndef MAX
#define MAX(a,b) (((a) > (b)) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a,b) (((a) < (b)) ? (a) : (b))
#endif

#define BIT(x) (1 << (x))
#define PRINT_BYTES(fd,s,len) do { \
	int __i,__n; \
	fprintf((fd),"     0  1  2  3  4  5  6  7  8  9  A  B  C  D  E  F\n"); \
	fprintf((fd),"     -----------------------------------------------\n"); \
	for (__i = 0, __n = 15;__i < len;__i++,__n--) { \
		if (__n == 15) \
			fprintf((fd),"%.2i - ",__i/16); \
		fprintf((fd),"%.2X ",(unsigned char)(s)[__i]); \
		if (__n == 0) { \
			fprintf((fd),"\n"); \
			__n = 16; \
		} \
	} \
	if (__n < 15) \
		fprintf((fd),"\n"); \
	fprintf((fd),"\n"); \
} while(0)


#ifndef alloca
#	ifdef __GNUC__
#		define alloca __builtin_alloca
#		define HAVE_ALLOCA 1
#	else
#		if defined HAVE_ALLOCA_H || defined _LIBC
#			include <alloca.h>
#		else
#			ifdef _AIX
#				pragma alloca
#			else
#				ifndef alloca
char *alloca ();
#				endif
#			endif
#		endif
#	endif
#endif /* ifndef alloca */
 

#endif /* _COMMON_H */
