/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/time.h>
#include <sys/types.h>
#include <openssl/bn.h>
#include <openssl/dh.h>

#define ERROR(...) printf(__VA_ARGS__)
#define WARN(...) printf(__VA_ARGS__)
#define DEBUG(...) printf(__VA_ARGS__)
#define INFO(...) printf(__VA_ARGS__)

#include "common.h"
#include "packet.h"
#include "link.h"

Link link_create() {
	Link link;
	bool link_readp(Link);
	void link_writep(Link);
	bool link_read(Link,void *,int);
	bool link_write(Link,void *,int);
	void link_free(Link);
	
	if ((link = (Link)malloc(sizeof(link_t))) == NULL) {
		ERROR("could not malloc space for a new link");
		return NULL;
	}

	link->readp  = &link_readp;
	link->writep = &link_writep;
	link->read   = &link_read;
	link->write  = &link_write;

	link->free = &link_free;

	return link;
}

void link_free(Link this) {
	free(this);
} 

bool link_readp(Link this) {
	ssize_t bytes = 0;
	size_t off    = 0;
	char *str;

	do {
		if ((bytes = read(this->sockfd,(void *)&this->recvbuf[off],(size_t)(PACKET_SIZE-off))) <= 0) {
			if (bytes == 0) {
				WARN("premature EOF on socket");
			} else {
				ERROR("socket read failed: %s",strerror(errno));
			}
			return false;
		}
		off += bytes;
	} while (off < PACKET_SIZE);

	/*str = hex(this->recvbuf,129);
	printf("In Packet:\n%s\n\n",str);
	free(str);*/

	if (this->recvbuf[0] != this->lflag) {
		WARN("received unexpected packet type");
		return false;
	}
	
	return true;
}

void link_writep(Link this) {
	ssize_t bytes = 0;
	size_t off    = 0;
	char *str;

	do {
		if ((bytes = write(this->sockfd,&this->sendbuf[off],(PACKET_SIZE-off))) < 0) {
			ERROR("failed to write packet: %s",strerror(errno));
			return;
		}
		off += bytes;
	} while (off < PACKET_SIZE);

}

bool link_read(Link this,void *buf,int len) {
	int i;

	for (i = PACKET_BOUNDARY(len)/PACKET_DATA_SIZE; i > 0; i--) {
		if (this->readp(this)) {
			memcpy(buf,&this->recvbuf[NODE_DATA_POS(this->hopid)],PACKET_DATA_SIZE);
			buf += PACKET_DATA_SIZE;
		} else {
			return false;
		}
	}
	return true;
}

bool link_write(Link this,void *buf,int len) {
	int i;

	this->sendbuf[0] = this->lflag;
	this->sendbuf[NODE_FLAG_POS(this->hopid)] = this->nflag;

	for (i = PACKET_BOUNDARY(len)/PACKET_DATA_SIZE; i > 0; i--) {
		memcpy(&this->sendbuf[NODE_DATA_POS(this->hopid)],buf,PACKET_DATA_SIZE);
		buf += PACKET_DATA_SIZE;
		/* RE-WRITE writep to return status */
		this->writep(this);
	}
	return true;
}
