/* ==========================================================================
 * AnonNet - Anonymous Network Project
 * --------------------------------------------------------------------------
 * Copyright (C) 2001  William Ahern
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 * ==========================================================================
 */

#ifndef _PACKET_H
#define _PACKET_H 1

#include "common.h"


#define NODE_MAX_P_SIZE		256
#define NODE_MAX_G_SIZE		256
#define NODE_MAX_Y_SIZE		256
#define NODE_SEED_SIZE		64
#define NODE_KEYMATERIAL_SIZE	(16 + 16 + 16 + 64 + 2 + 2)
#define NODE_CALLER_INITIATE_INPUT_SIZE (\
	2	/* cipher & hash */ +\
	3	/* p, g, and pubk sizes */ +\
	256	/* max p size */ +\
	256	/* max g size */ +\
	256	/* max pubk size */ +\
	64	/* max seed size */\
) /* caller must send exactly this much node bound data, no more, no less */

#define NODE_HASH_TIGER	0x00
#define NODE_HASH_SHA1	0x01

#define NODE_CIPHER_RIJNDAEL	0x01
#define NODE_CIPHER_TWOFISH	0x02

#define FLAG_CLR(f) ((f) = 0)
#define FLAG_SET(f,b) ((f) |= (b))
#define FLAG_ISSET(f,b) ((f) & (b))


#define PACKET_SIZE		129
#define PACKET_DATA_SIZE	48
#define PACKET_HEAD_SIZE	16


/*	Link Packet Flags
 *
 *	00 - NULL
 *	01 - NBND
 *	02 - INIT
 *	04 - RSRV
 *	08 - RSRV
 *	10 - RSRV
 *	20 - RSRV
 *	40 - RSRV
 *	80 - RSRV
 *	FF - KILL
 */
#define PACKET_LINK_NULL 0x00
#define PACKET_LINK_NBND 0x01
#define PACKET_LINK_INIT 0x02
#define PACKET_LINK_KILL 0xFF

/*	Virtual Link Packet Flags
 *
 *	00 - NULL
 *	01 - FWRD - Forward
 *	02 - INIT - Initiation
 *	04 - RSRV - Reserved
 *	08 - CNCT - Connect
 *	10 - RSRV - Reserved
 *	20 - RSRV - Reserved
 *	40 - RSRV - Reserved
 *	80 - RSRV - Reserved
 *	FF - KILL - Kill
 */
#define PACKET_NODE_NULL 0x00
#define PACKET_NODE_FWRD 0x01
#define PACKET_NODE_INIT 0x02
#define PACKET_NODE_CNCT 0x08
#define PACKET_NODE_KILL 0xFF

/*
 * these are relative to a 129 byte link packet, not the 128 byte
 * virtual tunnel packet
 */
#define PACKET_MAC_POS(hop)  (((hop)*16) + 1)
#define PACKET_SEQ_POS(hop)  (((hop)*16) + 11)
#define PACKET_FLAG_POS(hop) (((hop)*16) + 15)
#define PACKET_PLEN_POS(hop) (((hop)*16) + 16) /* payload length */

#define PACKET_ENC_BEG(hop) (((hop)*16) + 1)
#define PACKET_ENC_END(hop) 129
#define PACKET_ENC_LEN(hop) (PACKET_ENC_END(hop) - PACKET_ENC_BEG(hop))

#define PACKET_HEAD_BEG(hop) (((hop)*16) + 1)
#define PACKET_HEAD_LEN(hop) 16

#define PACKET_HASH_BEG(hop) (((hop)*16) + 11)
#define PACKET_HASH_END(hop) 129
#define PACKET_HASH_LEN(hop) (PACKET_HASH_END(hop) - PACKET_HASH_BEG(hop))
#define PACKET_DATA_POS(hop) 81 /* relative to Link buffer */
#define PACKET_BOUNDARY(n) ((n%PACKET_DATA_SIZE > 0)? (n+(PACKET_DATA_SIZE-(n%PACKET_DATA_SIZE))) : n)

/*
 * Connection Request Flags
 *
 * 	00 - RSRV
 * 	01 - IPV4
 * 	02 - IPV6
 * 	04 - NAME
 *	08 - RSRV
 * 	10 - RSRV
 * 	20 - RSRV
 * 	40 - RSRV
 * 	80 - RSRV
 * 	FF - RSRV
 */
#define PACKET_CNCT_REQ_IPV4 0x01
#define PACKET_CNCT_REQ_IPV6 0x02
#define PACKET_CNCT_REQ_NAME 0x04

/*
 * Connection Response Flags
 *
 * 	00 - FAIL
 * 	01 - IPV4
 * 	02 - IPV6
 * 	04 - RSRV
 *	08 - RSRV
 * 	10 - RSRV
 * 	20 - RSRV
 * 	40 - RSRV
 * 	80 - RSRV
 * 	FF - RSRV
 */
#define PACKET_CNCT_RSP_FAIL 0x00
#define PACKET_CNCT_RSP_IPV4 0x01
#define PACKET_CNCT_RSP_IPV6 0x02


#endif /* _PACKET_H */
