/*
 *    Copyright (C) 2001 Nikos Mavroyanopoulos
 *
 *    This library is free software; you can redistribute it and/or modify it 
 *    under the terms of the GNU Library General Public License as published 
 *    by the Free Software Foundation; either version 2 of the License, or 
 *    (at your option) any later version.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Library General Public License for more details.
 *
 *    You should have received a copy of the GNU Library General Public
 *    License along with this library; if not, write to the
 *    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *    Boston, MA 02111-1307, USA.
 */

#ifndef _TIGER_H
#define _TIGER_H

#include <config.h>

#include "common.h"


#define h0init 0x89ABCDEF;
#define h1init 0x01234567;
#define h2init 0x76543210;
#define h3init 0xFEDCBA98;
#define h4init 0xC3B2E187;
#define h5init 0xF096A5B4;

#define TIGER_DATALEN 16
#define TIGER_DIGESTLEN 6
#define TIGER128_DIGESTLEN 4
#define TIGER160_DIGESTLEN 5
#define TIGER_DIGESTSIZE 24
#define TIGER160_DIGESTSIZE 20
#define TIGER128_DIGESTSIZE 16
#define TIGER_DATASIZE 64

typedef struct tiger_ctx {
  word32 digest[TIGER_DIGESTLEN];  /* Message digest */ 
  word32 count_l, count_h;	 /* 64-bit block count */
  word8 block[TIGER_DATASIZE];     /* RIPEMD data buffer */  
  int index;                             /* index into buffer */
} TIGER_CTX;


void tiger_digest(struct tiger_ctx *ctx, word8 * s);
void tiger160_digest(struct tiger_ctx *ctx, word8 * s);
void tiger128_digest(struct tiger_ctx *ctx, word8 * s);
void tiger_final(struct tiger_ctx *ctx);
void tiger_update(struct tiger_ctx *ctx, word8 * buffer, word32 len);
void tiger_init(struct tiger_ctx *ctx);

#endif /* _TIGER_H */
